@@ -282,6 +282,27 @@ image_resize_to_layers_cmd_callback (GtkAction *action,
   gimp_image_flush (display->image);
 }
 
+void
+image_resize_to_selection_cmd_callback (GtkAction *action,
+                                        gpointer   data)
+{
+  GimpDisplay  *display;
+  GimpProgress *progress;
+  return_if_no_display (display, data);
+
+  progress = gimp_progress_start (GIMP_PROGRESS (display),
+                                  _("Resizing"), FALSE);
+
+  gimp_image_resize_to_selection (display->image,
+                                  action_data_get_context (data),
+                                  progress);
+
+  if (progress)
+    gimp_progress_end (progress);
+
+  gimp_image_flush (display->image);
+}
+
 void
 image_print_size_cmd_callback (GtkAction *action,
                                gpointer   data)
