@@ -66,6 +66,9 @@
 
 #define RESPONSE_RESET 1
 
+#define COLOR_BUTTON_WIDTH  40
+#define COLOR_BUTTON_HEIGHT 24
+
 
 /*  preferences local functions  */
 
@@ -969,6 +972,107 @@ prefs_table_new (gint          rows,
   return table;
 }
 
+static void
+prefs_profile_combo_dialog_response (GimpProfileChooserDialog *dialog,
+                                     gint                      response,
+                                     GimpColorProfileComboBox *combo)
+{
+  if (response == GTK_RESPONSE_ACCEPT)
+    {
+      gchar *filename;
+
+      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+
+      if (filename)
+        {
+          gchar *label = gimp_profile_chooser_dialog_get_desc (dialog,
+                                                               filename);
+
+          gimp_color_profile_combo_box_set_active (combo, filename, label);
+
+          g_free (label);
+        }
+    }
+
+  gtk_widget_hide (GTK_WIDGET (dialog));
+}
+
+static void
+prefs_profile_combo_changed (GimpColorProfileComboBox *combo,
+                             GObject                  *config)
+{
+  gchar *filename = gimp_color_profile_combo_box_get_active (combo);
+
+  g_object_set (config,
+                g_object_get_data (G_OBJECT (combo), "property-name"), filename,
+                NULL);
+
+  g_free (filename);
+}
+
+static GtkWidget *
+prefs_profile_combo_add_tooltip (GtkWidget   *combo,
+                                 GObject     *config,
+                                 const gchar *property_name)
+{
+  GParamSpec  *param_spec;
+  const gchar *blurb;
+
+  param_spec = g_object_class_find_property (G_OBJECT_GET_CLASS (config),
+                                             property_name);
+
+  blurb = g_param_spec_get_blurb (param_spec);
+
+  /*  can't set a tooltip on a combo_box  */
+  if (blurb)
+    {
+      GtkWidget *ebox = gtk_event_box_new ();
+
+      gimp_help_set_help_data (ebox,
+                               dgettext (GETTEXT_PACKAGE "-libgimp", blurb),
+                               NULL);
+      gtk_container_add (GTK_CONTAINER (ebox), combo);
+      gtk_widget_show (combo);
+
+      return ebox;
+    }
+
+  return combo;
+}
+
+static GtkWidget *
+prefs_profile_combo_box_new (Gimp         *gimp,
+                             GObject      *config,
+                             GtkListStore *store,
+                             const gchar  *label,
+                             const gchar  *property_name)
+{
+  GtkWidget *dialog = gimp_profile_chooser_dialog_new (gimp, label);
+  GtkWidget *combo;
+  gchar     *filename;
+
+  g_object_get (config, property_name, &filename, NULL);
+
+  combo = gimp_color_profile_combo_box_new_with_model (dialog,
+                                                       GTK_TREE_MODEL (store));
+
+  g_object_set_data (G_OBJECT (combo),
+                     "property-name", (gpointer) property_name);
+
+  gimp_color_profile_combo_box_set_active (GIMP_COLOR_PROFILE_COMBO_BOX (combo),
+                                           filename, NULL);
+
+  g_signal_connect (dialog, "response",
+                    G_CALLBACK (prefs_profile_combo_dialog_response),
+                    combo);
+
+  g_signal_connect (combo, "changed",
+                    G_CALLBACK (prefs_profile_combo_changed),
+                    config);
+
+  return prefs_profile_combo_add_tooltip (combo, config, property_name);
+}
+
 static GtkWidget *
 prefs_button_add (const gchar *stock_id,
                   const gchar *label,
@@ -1081,7 +1185,8 @@ prefs_color_button_add (GObject      *config,
 {
   GtkWidget *button = gimp_prop_color_button_new (config, property_name,
                                                   title,
-                                                  60, 24,
+                                                  COLOR_BUTTON_WIDTH,
+                                                  COLOR_BUTTON_HEIGHT,
                                                   GIMP_COLOR_AREA_FLAT);
 
   if (button)
@@ -1197,9 +1302,11 @@ prefs_display_options_frame_add (Gimp         *gimp,
   gtk_box_pack_start (GTK_BOX (hbox), checks_vbox, TRUE, TRUE, 0);
   gtk_widget_show (checks_vbox);
 
+#ifndef GDK_WINDOWING_QUARTZ
   prefs_check_button_add (object, "show-menubar",
                           _("Show _menubar"),
                           GTK_BOX (checks_vbox));
+#endif /* !GDK_WINDOWING_QUARTZ */
   prefs_check_button_add (object, "show-rulers",
                           _("Show _rulers"),
                           GTK_BOX (checks_vbox));
@@ -2294,7 +2401,7 @@ prefs_dialog_new (Gimp       *gimp,
                                      &top_iter,
                                      page_index++);
 
-  table = prefs_table_new (9, GTK_CONTAINER (vbox));
+  table = prefs_table_new (10, GTK_CONTAINER (vbox));
 
   {
     static const struct
@@ -2315,8 +2422,10 @@ prefs_dialog_new (Gimp       *gimp,
         N_("Select Printer Color Profile"), "printer-profile" }
     };
 
-    GObject *color_config;
-    gint     row = 0;
+    GObject      *color_config;
+    GtkListStore *store;
+    gchar        *filename;
+    gint          row = 0;
 
     g_object_get (object, "color-management", &color_config, NULL);
 
@@ -2325,16 +2434,19 @@ prefs_dialog_new (Gimp       *gimp,
                               GTK_TABLE (table), row++, NULL);
     gtk_table_set_row_spacing (GTK_TABLE (table), row - 1, 12);
 
+    filename = gimp_personal_rc_file ("profilerc");
+    store = gimp_color_profile_store_new (filename);
+    g_free (filename);
+
+    gimp_color_profile_store_add (GIMP_COLOR_PROFILE_STORE (store), NULL, NULL);
+
     for (i = 0; i < G_N_ELEMENTS (profiles); i++)
       {
-        GtkWidget *chooser;
-
-        chooser = gimp_profile_chooser_dialog_new (gimp,
-                                                   gettext (profiles[i].fs_label));
-        button =
-          gimp_prop_file_chooser_button_new_with_dialog (color_config,
-                                                         profiles[i].property_name,
-                                                         chooser);
+        button = prefs_profile_combo_box_new (gimp,
+                                              color_config,
+                                              store,
+                                              gettext (profiles[i].fs_label),
+                                              profiles[i].property_name);
 
         gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
                                    gettext (profiles[i].label), 0.0, 0.5,
@@ -2343,23 +2455,24 @@ prefs_dialog_new (Gimp       *gimp,
 
         if (i == 2) /* display profile */
           {
-#if defined (GDK_WINDOWING_X11)
+            gtk_table_set_row_spacing (GTK_TABLE (table), row - 2, 12);
+
             button =
               gimp_prop_check_button_new (color_config,
                                           "display-profile-from-gdk",
-                                          _("_Try to obtain the monitor "
-                                            "profile from the X server"));
+                                          _("_Try to use the system monitor "
+                                            "profile"));
 
             gtk_table_attach_defaults (GTK_TABLE (table),
-                                       button, 0, 2, row, row + 1);
-            row++;
+                                       button, 1, 2, row, row + 1);
             gtk_widget_show (button);
-#endif
+            row++;
 
             prefs_enum_combo_box_add (color_config,
                                       "display-rendering-intent", 0, 0,
                                       _("_Display rendering intent:"),
                                       GTK_TABLE (table), row++, NULL);
+            gtk_table_set_row_spacing (GTK_TABLE (table), row - 1, 12);
           }
 
         if (i == 3) /* printer profile */
@@ -2369,15 +2482,38 @@ prefs_dialog_new (Gimp       *gimp,
                                       _("_Softproof rendering intent:"),
                                       GTK_TABLE (table), row++, NULL);
           }
-
-        gtk_table_set_row_spacing (GTK_TABLE (table), row - 1, 12);
       }
 
-    g_object_unref (color_config);
+    g_object_unref (store);
+
+    hbox = gtk_hbox_new (FALSE, 6);
+    gtk_table_attach_defaults (GTK_TABLE (table), hbox, 1, 2, row, row + 1);
+    gtk_widget_show (hbox);
+    row++;
+
+    button = gimp_prop_check_button_new (color_config, "simulation-gamut-check",
+                                         _("Mark out of gamut colors"));
+    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
+    gtk_widget_show (button);
+
+    button = gimp_prop_color_button_new (color_config, "out-of-gamut-color",
+                                         _("Select Warning Color"),
+                                         COLOR_BUTTON_WIDTH,
+                                         COLOR_BUTTON_HEIGHT,
+                                         GIMP_COLOR_AREA_FLAT);
+    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+    gtk_widget_show (button);
+
+    gimp_color_panel_set_context (GIMP_COLOR_PANEL (button),
+                                  gimp_get_user_context (gimp));
+
+    gtk_table_set_row_spacing (GTK_TABLE (table), row - 1, 12);
 
     button = prefs_enum_combo_box_add (object, "color-profile-policy", 0, 0,
                                        _("File Open behaviour:"),
                                        GTK_TABLE (table), row++, NULL);
+
+    g_object_unref (color_config);
   }
 
 
