@@ -116,7 +116,9 @@ static gboolean       gui_pdb_dialog_set       (Gimp                *gimp,
 static gboolean       gui_pdb_dialog_close     (Gimp                *gimp,
                                                 GimpContainer       *container,
                                                 const gchar         *callback_name);
-
+static gboolean       gui_recent_list_add_uri  (Gimp                *gimp,
+                                                const gchar         *uri,
+                                                const gchar         *mime_type);
 
 /*  public functions  */
 
@@ -146,6 +148,7 @@ gui_vtable_init (Gimp *gimp)
   gimp->gui.pdb_dialog_new      = gui_pdb_dialog_new;
   gimp->gui.pdb_dialog_set      = gui_pdb_dialog_set;
   gimp->gui.pdb_dialog_close    = gui_pdb_dialog_close;
+  gimp->gui.recent_list_add_uri = gui_recent_list_add_uri;
 }
 
 
@@ -330,6 +333,14 @@ gui_free_progress (Gimp          *gimp,
     gtk_widget_destroy (GTK_WIDGET (progress));
 }
 
+static gboolean
+gui_pdb_dialog_present (GtkWindow *window)
+{
+  gtk_window_present (window);
+
+  return FALSE;
+}
+
 static gboolean
 gui_pdb_dialog_new (Gimp          *gimp,
                     GimpContext   *context,
@@ -427,6 +438,19 @@ gui_pdb_dialog_new (Gimp          *gimp,
 
           gtk_widget_show (dialog);
 
+          /*  workaround for bug #360106  */
+          {
+            GSource  *source = g_timeout_source_new (100);
+            GClosure *closure;
+
+            closure = g_cclosure_new_object (G_CALLBACK (gui_pdb_dialog_present),
+                                             G_OBJECT (dialog));
+
+            g_source_set_closure (source, closure);
+            g_source_attach (source, NULL);
+            g_source_unref (source);
+          }
+
           return TRUE;
         }
     }
@@ -519,3 +543,34 @@ gui_pdb_dialog_close (Gimp          *gimp,
 
   return FALSE;
 }
+
+static gboolean
+gui_recent_list_add_uri (Gimp        *gimp,
+                         const gchar *uri,
+                         const gchar *mime_type)
+{
+  GtkRecentManager *manager;
+  GtkRecentData     recent;
+  const gchar      *groups[2] = { "Graphics", NULL };
+
+  g_return_val_if_fail (GIMP_IS_GIMP (gimp), FALSE);
+  g_return_val_if_fail (uri != NULL, FALSE);
+
+  manager = gtk_recent_manager_get_default ();
+
+  g_return_val_if_fail (manager != NULL, FALSE);
+
+  /* use last part of the URI */
+  recent.display_name = NULL;
+
+  /* no special description */
+  recent.description  = NULL;
+  recent.mime_type    = (mime_type ?
+                         (gchar *) mime_type : "application/octet-stream");
+  recent.app_name     = "GNU Image Manipulation Program";
+  recent.app_exec     = GIMP_COMMAND " %u";
+  recent.groups       = (gchar **) groups;
+  recent.is_private   = FALSE;
+
+  return gtk_recent_manager_add_full (manager, uri, &recent);
+}
