@@ -150,9 +150,8 @@ _gimp_paintbrush_motion (GimpPaintCore    *paint_core,
   /* otherwise check if the brush has a pixmap and use that to color the area */
   else if (brush_core->brush && brush_core->brush->pixmap)
     {
-      gimp_brush_core_color_area_with_pixmap (brush_core, image, drawable,
+      gimp_brush_core_color_area_with_pixmap (brush_core, drawable,
                                               area,
-                                              brush_core->scale,
                                               gimp_paint_options_get_brush_mode (paint_options));
 
       paint_appl_mode = GIMP_PAINT_INCREMENTAL;
