@@ -112,6 +112,9 @@ static void     gimp_rect_select_tool_real_select         (GimpRectSelectTool *r
                                                            gint                y,
                                                            gint                w,
                                                            gint                h);
+static void     gimp_rect_select_tool_update_option_defaults
+                                                          (GimpRectSelectTool *rect_select_tool,
+                                                           gboolean            ignore_pending);
 
 static void    gimp_rect_select_tool_round_corners_notify (GimpRectSelectOptions *options,
                                                            GParamSpec            *pspec,
@@ -220,6 +223,9 @@ gimp_rect_select_tool_constructor (GType                  type,
                            G_CALLBACK (gimp_rect_select_tool_round_corners_notify),
                            object, 0);
 
+  gimp_rect_select_tool_update_option_defaults (GIMP_RECT_SELECT_TOOL (object),
+                                                FALSE);
+
   return object;
 }
 
@@ -293,6 +299,7 @@ gimp_rect_select_tool_button_press (GimpTool        *tool,
 {
   GimpRectangleTool     *rectangle   = GIMP_RECTANGLE_TOOL (tool);
   GimpRectSelectTool    *rect_select = GIMP_RECT_SELECT_TOOL (tool);
+  GimpDisplayShell      *shell       = GIMP_DISPLAY_SHELL (display->shell);
   GimpRectangleFunction  function;
 
   if (tool->display && display != tool->display)
@@ -300,15 +307,14 @@ gimp_rect_select_tool_button_press (GimpTool        *tool,
 
   function = gimp_rectangle_tool_get_function (rectangle);
 
-  rect_select->saved_show_selection
-    = gimp_display_shell_get_show_selection (GIMP_DISPLAY_SHELL (display->shell));
+  rect_select->saved_show_selection =
+    gimp_display_shell_get_show_selection (shell);
 
   if (function == RECT_INACTIVE)
     {
-      GimpDisplay *old_display;
+      GimpDisplay *old_display = tool->display;
       gboolean     edit_started;
 
-      old_display = tool->display;
       tool->display = display;
       gimp_tool_control_activate (tool->control);
 
@@ -317,6 +323,7 @@ gimp_rect_select_tool_button_press (GimpTool        *tool,
 
       if (gimp_tool_control_is_active (tool->control))
         gimp_tool_control_halt (tool->control);
+
       tool->display = old_display;
 
       if (edit_started)
@@ -370,8 +377,7 @@ gimp_rect_select_tool_button_press (GimpTool        *tool,
       /* if the operation is "Replace", turn off the marching ants,
          because they are confusing */
       if (operation == GIMP_CHANNEL_OP_REPLACE)
-        gimp_display_shell_set_show_selection (GIMP_DISPLAY_SHELL (display->shell),
-                                               FALSE);
+        gimp_display_shell_set_show_selection (shell, FALSE);
     }
 
   rect_select->undo = NULL;
@@ -398,9 +404,7 @@ gimp_rect_select_tool_button_release (GimpTool              *tool,
   if (release_type == GIMP_BUTTON_RELEASE_CLICK)
     {
       GimpImage *image = tool->display->image;
-      GimpUndo  *redo;
-
-      redo = gimp_undo_stack_peek (image->redo_stack);
+      GimpUndo  *redo  = gimp_undo_stack_peek (image->redo_stack);
 
       if (redo && rect_select->redo == redo)
         {
@@ -433,6 +437,9 @@ gimp_rect_select_tool_button_release (GimpTool              *tool,
     }
 
   rect_select->redo = NULL;
+
+  gimp_rect_select_tool_update_option_defaults (rect_select,
+                                                FALSE);
 }
 
 static void
@@ -505,14 +512,12 @@ gimp_rect_select_tool_select (GimpRectangleTool *rectangle,
   GimpTool             *tool        = GIMP_TOOL (rectangle);
   GimpRectSelectTool   *rect_select = GIMP_RECT_SELECT_TOOL (rectangle);
   GimpSelectionOptions *options     = GIMP_SELECTION_TOOL_GET_OPTIONS (tool);
-  GimpImage            *image;
+  GimpImage            *image       = tool->display->image;
   gboolean              rectangle_exists;
   GimpChannelOps        operation;
 
   gimp_tool_pop_status (tool, tool->display);
 
-  image = tool->display->image;
-
   rectangle_exists = (x <= image->width && y <= image->height &&
                       x + w >= 0 && y + h >= 0 &&
                       w > 0 && h > 0);
@@ -539,20 +544,24 @@ gimp_rect_select_tool_real_select (GimpRectSelectTool *rect_select,
                                    gint                w,
                                    gint                h)
 {
-  GimpTool              *tool = GIMP_TOOL (rect_select);
-  GimpSelectionOptions  *options;
+  GimpTool              *tool    = GIMP_TOOL (rect_select);
+  GimpSelectionOptions  *options = GIMP_SELECTION_TOOL_GET_OPTIONS (tool);
   GimpRectSelectOptions *rect_select_options;
+  GimpChannel           *channel;
 
-  options             = GIMP_SELECTION_TOOL_GET_OPTIONS (tool);
   rect_select_options = GIMP_RECT_SELECT_TOOL_GET_OPTIONS (tool);
 
+  channel = gimp_image_get_mask (tool->display->image);
+
   if (rect_select_options->round_corners)
     {
-      /* To prevent elliptification of the rect, we must cap the corner radius */
-      gdouble radius = MIN (rect_select_options->corner_radius,
-                            MIN (w / 2.0, h / 2.0));
+      /* To prevent elliptification of the rectangle,
+       * we must cap the corner radius.
+       */
+      gdouble max    = MIN (w / 2.0, h / 2.0);
+      gdouble radius = MIN (rect_select_options->corner_radius, max);
 
-      gimp_channel_select_round_rect (gimp_image_get_mask (tool->display->image),
+      gimp_channel_select_round_rect (channel,
                                       x, y, w, h,
                                       radius, radius,
                                       operation,
@@ -564,7 +573,7 @@ gimp_rect_select_tool_real_select (GimpRectSelectTool *rect_select,
     }
   else
     {
-      gimp_channel_select_rectangle (gimp_image_get_mask (tool->display->image),
+      gimp_channel_select_rectangle (channel,
                                      x, y, w, h,
                                      operation,
                                      options->feather,
@@ -574,6 +583,55 @@ gimp_rect_select_tool_real_select (GimpRectSelectTool *rect_select,
     }
 }
 
+/**
+ * gimp_rect_select_tool_update_option_defaults:
+ * @crop_tool:
+ * @ignore_pending: %TRUE to ignore any pending crop rectangle.
+ *
+ * Sets the default Fixed: Aspect ratio and Fixed: Size option
+ * properties.
+ */
+static void
+gimp_rect_select_tool_update_option_defaults (GimpRectSelectTool *rect_select_tool,
+                                              gboolean            ignore_pending)
+{
+  GimpTool             *tool;
+  GimpRectangleTool    *rectangle_tool;
+  GimpRectangleOptions *rectangle_options;
+
+  tool              = GIMP_TOOL (rect_select_tool);
+  rectangle_tool    = GIMP_RECTANGLE_TOOL (tool);
+  rectangle_options = GIMP_RECTANGLE_TOOL_GET_OPTIONS (rectangle_tool);
+
+  if (tool->display != NULL && !ignore_pending)
+    {
+      /* There is a pending rectangle and we should not ignore it, so
+       * set default Fixed: Size to the same as the current pending
+       * rectangle width/height.
+       */
+
+      gimp_rectangle_tool_pending_size_set (rectangle_tool,
+                                            G_OBJECT (rectangle_options),
+                                            "default-aspect-numerator",
+                                            "default-aspect-denominator");
+
+      g_object_set (G_OBJECT (rectangle_options),
+                    "use-string-current", TRUE,
+                    NULL);
+    }
+  else
+    {
+      g_object_set (G_OBJECT (rectangle_options),
+                    "default-aspect-numerator",   1.0,
+                    "default-aspect-denominator", 1.0,
+                    NULL);
+
+      g_object_set (G_OBJECT (rectangle_options),
+                    "use-string-current", FALSE,
+                    NULL);
+    }
+}
+
 /*
  * This function is called if the user clicks and releases the left
  * button without moving it.  There are the things we might want
@@ -647,6 +705,9 @@ gimp_rect_select_tool_execute (GimpRectangleTool *rectangle,
         }
     }
 
+  gimp_rect_select_tool_update_option_defaults (GIMP_RECT_SELECT_TOOL (rectangle),
+                                                FALSE);
+
   return TRUE;
 }
 
@@ -665,7 +726,7 @@ gimp_rect_select_tool_cancel (GimpRectangleTool *rectangle)
        * we have already "executed", and need to undo at this point,
        * unless the user has done something in the meantime
        */
-      undo = gimp_undo_stack_peek (image->undo_stack);
+      undo  = gimp_undo_stack_peek (image->undo_stack);
 
       if (undo && rect_select->undo == undo)
         {
@@ -728,7 +789,9 @@ gimp_rect_select_tool_rectangle_changed (GimpRectangleTool *rectangle)
 
       if (! rect_select->use_saved_op)
         {
-          GimpSelectionOptions *options = GIMP_SELECTION_TOOL_GET_OPTIONS (tool);
+          GimpSelectionOptions *options;
+
+          options = GIMP_SELECTION_TOOL_GET_OPTIONS (tool);
 
           /* remember the operation now in case we modify the rectangle */
           rect_select->operation    = options->operation;
