@@ -47,6 +47,19 @@ struct _GimpColorConfig
   GimpColorRenderingIntent    simulation_intent;
 
   gchar                      *display_module;
+
+  gboolean                    simulation_gamut_check;
+  GimpRGB                     out_of_gamut_color;
+
+  /* Padding for future expansion */
+  void (* _gimp_reserved1) (void);
+  void (* _gimp_reserved2) (void);
+  void (* _gimp_reserved3) (void);
+  void (* _gimp_reserved4) (void);
+  void (* _gimp_reserved5) (void);
+  void (* _gimp_reserved6) (void);
+  void (* _gimp_reserved7) (void);
+  void (* _gimp_reserved8) (void);
 };
 
 struct _GimpColorConfigClass
