@@ -47,6 +47,22 @@
 #include <errno.h>
 #include <string.h>
 
+#include <sys/types.h>
+#include <fcntl.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <glib/gstdio.h>
+#ifdef G_OS_WIN32
+#include <libgimpbase/gimpwin32-io.h>
+#endif
+
+#ifndef _O_BINARY
+#define _O_BINARY 0
+#endif
+
 #include <tiffio.h>
 
 #include <libgimp/gimp.h>
@@ -167,14 +183,15 @@ static void      read_default  (const guchar *source,
                                 gint          extra,
                                 gint          align);
 
-static void      fill_bit2byte          (void);
+static void      fill_bit2byte (void);
+
+static void      tiff_warning  (const gchar  *module,
+                                const gchar  *fmt,
+                                va_list       ap);
+static void      tiff_error    (const gchar  *module,
+                                const gchar  *fmt,
+                                va_list       ap);
 
-static void      tiff_warning           (const gchar *module,
-                                         const gchar *fmt,
-                                         va_list      ap);
-static void      tiff_error             (const gchar *module,
-                                         const gchar *fmt,
-                                         va_list      ap);
 
 const GimpPlugInInfo PLUG_IN_INFO =
 {
@@ -260,26 +277,30 @@ run (const gchar      *name,
 
   if (strcmp (name, LOAD_PROC) == 0)
     {
-      TIFF       *tif;
+      const gchar *filename = param[1].data.d_string;
+      TIFF        *tif;
+      gint         fd;
 
-      gimp_get_data (LOAD_PROC, &target);
+      fd = g_open (filename, O_RDONLY | _O_BINARY, 0);
 
-      tif = TIFFOpen (param[1].data.d_string, "r");
-
-      if (! tif)
+      if (fd == -1)
         {
           g_message (_("Could not open '%s' for reading: %s"),
-                     gimp_filename_to_utf8 (param[1].data.d_string), g_strerror (errno));
+                     gimp_filename_to_utf8 (filename), g_strerror (errno));
           status = GIMP_PDB_EXECUTION_ERROR;
         }
       else
         {
+          tif = TIFFFdOpen (fd, filename, "r");
+
+          gimp_get_data (LOAD_PROC, &target);
+
           pages.n_pages = pages.o_pages = TIFFNumberOfDirectories (tif);
 
           if (pages.n_pages == 0)
             {
               g_message (_("TIFF '%s' does not contain any directories"),
-                         gimp_filename_to_utf8 (param[1].data.d_string));
+                         gimp_filename_to_utf8 (filename));
 
               status = GIMP_PDB_EXECUTION_ERROR;
             }
@@ -331,7 +352,9 @@ run (const gchar      *name,
               else
                 status = GIMP_PDB_CANCEL;
             }
+
           TIFFClose (tif);
+          close (fd);
         }
     }
   else
@@ -719,7 +742,9 @@ load_image (const gchar       *filename,
        * that can handle ICC profiles. Otherwise just ignore this section. */
       if (TIFFGetField (tif, TIFFTAG_ICCPROFILE, &profile_size, &icc_profile))
         {
-          parasite = gimp_parasite_new ("icc-profile", 0,
+          parasite = gimp_parasite_new ("icc-profile",
+                                        GIMP_PARASITE_PERSISTENT |
+                                        GIMP_PARASITE_UNDOABLE,
                                         profile_size, icc_profile);
           gimp_image_parasite_attach (image, parasite);
           gimp_parasite_free (parasite);
@@ -1111,9 +1136,10 @@ load_rgba (TIFF         *tif,
       for (i = rowStart; i < rowEnd; i++)
         buffer[i] = GUINT32_TO_LE (buffer[i]);
 #endif
-      gimp_pixel_rgn_set_rect (&(channel[0].pixel_rgn),
-                               channel[0].pixels + row * imageWidth * 4,
-                               0, imageLength -row -1, imageWidth, 1);
+
+      gimp_pixel_rgn_set_row (&(channel[0].pixel_rgn),
+                              channel[0].pixels + row * imageWidth * 4,
+                              0, imageLength -row -1, imageWidth);
 
       if ((row % 32) == 0)
         gimp_progress_update ((gdouble) row / (gdouble) imageLength);
