@@ -67,9 +67,10 @@
 
 typedef struct
 {
-  gint32  width;
-  gint32  height;
-  gint32  paned_position;
+  gint     width;
+  gint     height;
+  gint     paned_position;
+  gdouble  zoom;
 } DialogData;
 
 enum
@@ -136,7 +137,7 @@ static void       link_clicked       (HtmlDocument      *doc,
                                       gpointer           data);
 static gboolean   request_url        (HtmlDocument      *doc,
                                       const gchar       *uri,
-                                      HtmlStream       *stream,
+                                      HtmlStream        *stream,
                                       GError            **error);
 static gboolean   io_handler         (GIOChannel        *io,
                                       GIOCondition       condition,
@@ -155,19 +156,20 @@ static gchar    * filename_from_uri  (const gchar       *uri);
 
 /*  private variables  */
 
-static const gchar   *eek_png_tag  = "<h1>Eeek!</h1>";
+static const gchar   *eek_png_tag    = "<h1>Eeek!</h1>";
 
-static Queue         *queue        = NULL;
-static gchar         *current_uri  = NULL;
+static Queue         *queue          = NULL;
+static gchar         *current_uri    = NULL;
+static GHashTable    *uri_hash_table = NULL;
 
-static GtkWidget     *html         = NULL;
-static GtkWidget     *tree_view    = NULL;
-static GtkUIManager  *ui_manager   = NULL;
-static GtkWidget     *button_prev  = NULL;
-static GtkWidget     *button_next  = NULL;
-static GdkCursor     *busy_cursor  = NULL;
+static GtkWidget     *html           = NULL;
+static GtkWidget     *tree_view      = NULL;
+static GtkUIManager  *ui_manager     = NULL;
+static GtkWidget     *button_prev    = NULL;
+static GtkWidget     *button_next    = NULL;
+static GdkCursor     *busy_cursor    = NULL;
 
-static GtkTargetEntry help_dnd_target_table[] =
+static const GtkTargetEntry help_dnd_target_table[] =
 {
   { "text/uri-list", 0, 0 },
   { "_NETSCAPE_URL", 0, 0 }
@@ -195,7 +197,7 @@ browser_dialog_open (void)
   GtkCellRenderer *cell;
   GdkPixbuf       *pixbuf;
   gchar           *eek_png_path;
-  DialogData       data = { 720, 560, 240 };
+  DialogData       data = { 720, 560, 240, 1.0 };
 
   gimp_ui_init ("helpbrowser", TRUE);
 
@@ -377,6 +379,7 @@ browser_dialog_open (void)
                     G_CALLBACK (view_button_press),
                     NULL);
 
+  html_view_set_magnification (HTML_VIEW (html), data.zoom);
   html_view_set_document (HTML_VIEW (html), html_document_new ());
 
   g_signal_connect (HTML_VIEW (html)->document, "title-changed",
@@ -462,6 +465,9 @@ browser_dialog_load (const gchar *uri,
       anchor = NULL;
     }
 
+  if (uri_hash_table)
+    browser_dialog_goto_index (new_uri);
+
   if (! has_case_prefix (abs, "file:/"))
     {
       load_remote_page (uri);
@@ -571,12 +577,15 @@ help_item_compare (gconstpointer a,
 }
 
 static void
-add_child (GtkTreeStore *store,
-           GtkTreeIter  *parent,
-           GimpHelpItem *item)
+add_child (GtkTreeStore   *store,
+           GimpHelpDomain *domain,
+           GimpHelpLocale *locale,
+           GtkTreeIter    *parent,
+           GimpHelpItem   *item)
 {
   GtkTreeIter  iter;
   GList       *list;
+  gchar       *uri;
 
   gtk_tree_store_append (store, &iter, parent);
 
@@ -585,13 +594,22 @@ add_child (GtkTreeStore *store,
                       1, item->title,
                       -1);
 
+  uri = g_strconcat (domain->help_uri,  "/",
+                     locale->locale_id, "/",
+                     item->ref,
+                     NULL);
+
+  g_hash_table_insert (uri_hash_table,
+                       uri,
+                       gtk_tree_iter_copy (&iter));
+
   item->children = g_list_sort (item->children, help_item_compare);
 
   for (list = item->children; list; list = g_list_next (list))
     {
       GimpHelpItem *item = list->data;
 
-      add_child (store, &iter, item);
+      add_child (store, domain, locale, &iter, item);
     }
 }
 
@@ -619,11 +637,19 @@ browser_dialog_make_index (GimpHelpDomain *domain,
   g_object_set_data (G_OBJECT (store), "domain", domain);
   g_object_set_data (G_OBJECT (store), "locale", locale);
 
+  if (uri_hash_table)
+    g_hash_table_unref (uri_hash_table);
+
+  uri_hash_table = g_hash_table_new_full (g_str_hash,
+                                          g_str_equal,
+                                          (GDestroyNotify) g_free,
+                                          (GDestroyNotify) gtk_tree_iter_free);
+
   for (list = locale->toplevel_items; list; list = g_list_next (list))
     {
       GimpHelpItem *item = list->data;
 
-      add_child (store, NULL, item);
+      add_child (store, domain, locale, NULL, item);
     }
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view),
@@ -631,6 +657,38 @@ browser_dialog_make_index (GimpHelpDomain *domain,
   g_object_unref (store);
 }
 
+void
+browser_dialog_goto_index (const gchar *ref)
+{
+  GtkTreeIter *iter;
+
+  iter = g_hash_table_lookup (uri_hash_table, ref);
+
+  if (iter)
+    {
+      GtkTreeSelection *selection;
+      GtkTreeModel     *model;
+      GtkTreePath      *path;
+      GtkTreePath      *scroll_path;
+
+      selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+      model     = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
+
+      path = gtk_tree_model_get_path (model, iter);
+      scroll_path = gtk_tree_path_copy (path);
+
+      gtk_tree_path_up (path);
+      gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
+      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view), scroll_path,
+                                    NULL, FALSE, 0.0, 0.0);
+
+      gtk_tree_path_free (path);
+      gtk_tree_path_free (scroll_path);
+
+      gtk_tree_selection_select_iter (selection, iter);
+    }
+}
+
 
 /*  private functions  */
 
@@ -821,8 +879,8 @@ index_callback (GtkAction *action,
 }
 
 static void
-zoom_in_callback (GtkAction  *action,
-                  gpointer  data)
+zoom_in_callback (GtkAction *action,
+                  gpointer   data)
 {
   html_view_zoom_in (HTML_VIEW (html));
 }
@@ -997,15 +1055,13 @@ dialog_unmap (GtkWidget *window,
               GtkWidget *paned)
 {
   DialogData  data;
-  gint        width;
-  gint        height;
 
-  gtk_window_get_size (GTK_WINDOW (window), &width, &height);
+  gtk_window_get_size (GTK_WINDOW (window), &data.width, &data.height);
 
-  data.width          = width;
-  data.height         = height;
   data.paned_position = gtk_paned_get_position (GTK_PANED (paned));
 
+  data.zoom = html ? html_view_get_magnification (HTML_VIEW (html)) : 1.0;
+
   gimp_set_data (GIMP_HELP_BROWSER_DIALOG_DATA, &data, sizeof (data));
 
   gtk_main_quit ();
