@@ -36,7 +36,6 @@ struct cell {
   union {
     struct {
       char   *_svalue;
-      char   *_skey;
       int   _length;
     } _string;
     num _number;
@@ -58,8 +57,8 @@ func_dealloc free;
 int retcode;
 int tracing;
 
-#define CELL_SEGSIZE    5000  /* # of cells in one segment */
-#define CELL_NSEGMENT   20    /* # of segments for cells */
+#define CELL_SEGSIZE    25000 /* # of cells in one segment */
+#define CELL_NSEGMENT   50    /* # of segments for cells */
 char *alloc_seg[CELL_NSEGMENT];
 pointer cell_seg[CELL_NSEGMENT];
 int     last_cell_seg;
@@ -70,10 +69,10 @@ pointer envir;           /* stack register for current environment */
 pointer code;            /* register for current code */
 pointer dump;            /* stack register for next evaluation */
 pointer safe_foreign;    /* register to avoid gc problems */
+pointer foreign_error;   /* used for foreign functions to signal an error */
 
 int interactive_repl;    /* are we in an interactive REPL? */
 int print_output;        /* set to 1 to print results and error messages */
-int print_error;         /* set to 1 while printing error messages */
 
 struct cell _sink;
 pointer sink;            /* when mem. alloc. fails */
@@ -118,7 +117,6 @@ char    gc_verbose;      /* if gc_verbose is not zero, print gc status */
 char    no_memory;       /* Whether mem. alloc. has failed */
 
 #define LINESIZE 1024
-char    linebuff[LINESIZE];
 char    strbuff[LINESIZE];
 
 FILE *tmpfp;
